
Public Class MonthViewSingleAppointmentCreationFilter
    Implements IUIElementCreationFilter

    Private DayApps As New Dictionary(Of Appointment, SingleDayAppointmentUIElement)()
    Private ulMonthViewSingle As Infragistics.Win.UltraWinSchedule.UltraMonthViewSingle

    Shared iAppWidth As Integer

    Public Sub New(ByVal ultraMonthViewSingle1 As Infragistics.Win.UltraWinSchedule.UltraMonthViewSingle)
        Me.ulMonthViewSingle = ultraMonthViewSingle1
        ultraMonthViewSingle1.DrawFilter = New MonthViewSingleAppointmentDrawFilter(DayApps)
    End Sub

    Public Sub AfterCreateChildElements(ByVal parent As UIElement) Implements IUIElementCreationFilter.AfterCreateChildElements
        If TypeOf parent Is DayUIElement Then
            Dim objDayElement As DayUIElement = DirectCast(parent, DayUIElement)
            If objDayElement.Day.Appointments.Count > 1 Then
                objDayElement.MoreActivityIndicatorVisible = True
            End If
        End If      
    End Sub

    Public Function BeforeCreateChildElements(ByVal parent As UIElement) As Boolean Implements IUIElementCreationFilter.BeforeCreateChildElements
        Dim dayHeader As DayNumberUIElement = TryCast(parent, DayNumberUIElement)
        If dayHeader IsNot Nothing Then
            Dim day As DayUIElement = TryCast(dayHeader.Parent, DayUIElement)

            If day.Day.Appointments.Count = 0 Then
                Return False
            End If

            Dim AppAreaLocation As Point = dayHeader.Rect.Location
            AppAreaLocation.Offset(0, dayHeader.Rect.Height + 2)

            Dim sFirstHalf As String = String.Empty
            Dim sSecondHalf As String = String.Empty

            sFirstHalf = day.Day.Appointments(0).Text.Substring(0, day.Day.Appointments(0).Text.Split(" ")(0).Length + 3).Trim()
            sSecondHalf = day.Day.Appointments(0).Text.Substring(sFirstHalf.Length, day.Day.Appointments(0).Text.Length - sFirstHalf.Length).Trim()
            day.Day.Appointments(0).Subject = sFirstHalf + vbCrLf + sSecondHalf

            Dim objFirstApp As New SingleDayAppointmentUIElement(day, day.Day.Appointments(0))
            Dim objAppFont As New Font("Arial", 20, FontStyle.Regular, GraphicsUnit.Point)
            Dim objSizeF As SizeF = TextRenderer.MeasureText(day.Day.Appointments(0).Subject, objAppFont)
            Dim iHeight As Integer = IIf(CInt(objSizeF.Height) <= 0, 35, CInt(objSizeF.Height))
            Dim appSize As New Size(IIf(objSizeF.Width > 105, 105, objSizeF.Width), iHeight)

            AppAreaLocation = New Point(dayHeader.Rect.X + (dayHeader.Rect.Width / 2) - (appSize.Width / 2) + 2, AppAreaLocation.Y)

            Dim AppArea As New Size(dayHeader.Rect.Width, day.Rect.Height - dayHeader.Rect.Height - 2)
            Dim appLocation As Point = AppAreaLocation
            Dim newAppUI As New List(Of SingleDayAppointmentUIElement)()

            For Each app As Appointment In day.Day.Appointments
                Dim appUI As New SingleDayAppointmentUIElement(day, app)

                sFirstHalf = app.Text.Substring(0, app.Text.Split(" ")(0).Length + 3).Trim()
                sSecondHalf = app.Text.Substring(sFirstHalf.Length, app.Text.Length - sFirstHalf.Length).Trim()
                app.Subject = sFirstHalf + vbCrLf + sSecondHalf

                objAppFont = New Font("Arial", 20, FontStyle.Regular, GraphicsUnit.Point)
                objSizeF = TextRenderer.MeasureText(app.Subject, objAppFont)
                iHeight = IIf(CInt(objSizeF.Height) <= 0, 30, CInt(objSizeF.Height))
                appSize = New Size(IIf(objSizeF.Width > 105, 105, objSizeF.Width), iHeight - 10)

                AppAreaLocation = New Point(dayHeader.Rect.X + (dayHeader.Rect.Width / 2) - (appSize.Width / 2) + 2, AppAreaLocation.Y)
                appLocation = New Point(appLocation.X, appLocation.Y)
                appUI.Rect = New Rectangle(appLocation, appSize)

                If day.Day.Appointments.Count > 1 Then
                    day.MoreActivityIndicatorVisible = True
                    DirectCast(DirectCast(appUI.Parent, Infragistics.Win.UIElement), Infragistics.Win.UltraWinSchedule.MonthViewSingle.DayUIElement).MoreActivityIndicatorVisible = True
                End If

                If DayApps.ContainsKey(app) Then
                    DayApps(app) = appUI
                Else
                    DayApps.Add(app, appUI)
                End If
                newAppUI.Add(appUI)

                If appLocation.X + appSize.Width * 2 > AppAreaLocation.X + AppArea.Width Then
                    appLocation = New Point(AppAreaLocation.X, appLocation.Y + appSize.Height)
                Else
                    appLocation.Offset(appSize.Width, 0)
                End If
            Next
            day.ChildElements.AddRange(newAppUI)

            Dim sel As AppointmentSelectedUIElement = TryCast(parent, AppointmentSelectedUIElement)
            If sel IsNot Nothing Then
                Return True
            End If
            Return False
        ElseIf TypeOf parent Is EditorWithTextDisplayTextUIElement Then
            Dim editor As EditorWithTextDisplayTextUIElement = DirectCast(parent, EditorWithTextDisplayTextUIElement)
            editor.WrapText = True
            editor.MultiLine = True

            Dim objAppFont As New Font("Arial", 20, FontStyle.Regular, GraphicsUnit.Point)
            Dim objSizeF As SizeF = TextRenderer.MeasureText(editor.Text, objAppFont)
            Dim iHeight As Integer = IIf(CInt(objSizeF.Height) <= 0, 24, CInt(objSizeF.Height))
            Dim appSize As New Size(IIf(objSizeF.Width > 105, 105, objSizeF.Width), iHeight)

            Dim iX As Integer = editor.Parent.Parent.Parent.Rect.X
            editor.Rect = New Rectangle(editor.Parent.Parent.Parent.Rect.X + (editor.Parent.Parent.Parent.Rect.Width / 2) - (editor.Rect.Width / 2) + 8, editor.Rect.Y, appSize.Width, iHeight)
        End If
        Return False
    End Function

    Private Function GetTextHeight(ByVal objAppointment As Appointment) As Integer
        Dim fontToMeasure As New Font("Microsoft Sans Serif", 14, FontStyle.Bold)
        Dim sizeOfString As New SizeF
        Dim g As Graphics

        sizeOfString = g.MeasureString("This is line 1" & vbCrLf & "This is line 2", fontToMeasure)
        Debug.WriteLine("String Height: " & sizeOfString.Height)
        Debug.WriteLine("String Width: " & sizeOfString.Width)

        Return 0
    End Function

End Class


Class MonthViewSingleAppointmentDrawFilter
    Implements IUIElementDrawFilter
    Private DayApps As Dictionary(Of Appointment, SingleDayAppointmentUIElement)
    Public Sub New(ByRef DayApps As Dictionary(Of Appointment, SingleDayAppointmentUIElement))
        Me.DayApps = DayApps
    End Sub

    Public Function DrawElement(ByVal drawPhase As DrawPhase, ByRef drawParams As UIElementDrawParams) As Boolean Implements IUIElementDrawFilter.DrawElement
        Return True
    End Function

    Public Function GetPhasesToFilter(ByRef drawParams As UIElementDrawParams) As DrawPhase Implements IUIElementDrawFilter.GetPhasesToFilter
        Dim phase As DrawPhase = DrawPhase.None
        If TypeOf drawParams.Element Is SingleDayAppointmentUIElement Then
            Dim app As SingleDayAppointmentUIElement = TryCast(drawParams.Element, SingleDayAppointmentUIElement)

            If app.Appointment IsNot Nothing AndAlso DayApps.Count > 0 AndAlso Not DayApps(app.Appointment).Equals(app) Then
                phase = DrawPhase.BeforeDrawElement
            End If
        End If
        Return phase
    End Function
End Class